#ifndef H_INT_COEFFICIENT_FUNCTION
# define H_INT_COEFFICIENT_FUNCTION
#include "comp.hpp"
#include "ArgumentMarker.hpp"
#include <iostream>
#include <cmath>
#include <cassert>

using namespace ngcomp;

namespace ngfem 
{
	class IntCoefficientFunction : public CoefficientFunction 
	{
		std::shared_ptr<ngcomp::MeshAccess> mesh;
		std::shared_ptr<CoefficientFunction> cf;

	public:
		IntCoefficientFunction(
			std::shared_ptr<ngcomp::MeshAccess> mesh,
			std::shared_ptr<CoefficientFunction> cf
		) : CoefficientFunction(1)
		{
			this->mesh = mesh; 
			this->cf = cf;
		}

		virtual double Evaluate(const BaseMappedIntegrationPoint& mip) const override
		{
			size_t global_heapsize = 1000000;
			LocalHeap glh(global_heapsize, "python-comp lh", true);

			// set the marker for the integration points
			FlatVector<> point = mip.GetPoint();
			ArgumentMarkerData amd = {
				.x = point[0],
				.y = point[1]
			};
			// do integration over whole domain
			int dim = this->cf->Dimension(); // in case of vector-valued cf
			Vector<> sum(dim);
			sum = 0.0; // init to zero component-wise
			this->mesh->IterateElements(
				VOL, 
				glh,
				[&] (Ngs_Element el, LocalHeap& lh)
				{
					// transformation into
					// physical coords
					ElementTransformation& trafo = this->mesh->GetTrafo(el, lh); 
					// pass coordinates down to ArgumentMarkers
					trafo.userdata = reinterpret_cast<void*>(&amd);
					
				   // holds int approx on el
				   // for every component of cf (1 in our case)
					FlatVector<> hsum(dim, lh); 
					hsum = 0.0; 

					IntegrationRule ir(trafo.GetElementType(), 2); // quad points
					BaseMappedIntegrationRule& mir = trafo(ir, lh); // transf quad points
					FlatMatrix<> values(ir.Size(), dim, lh);
					this->cf->Evaluate(mir, values); // apply on all transf quad points
					// over all integration points, every component of a row
					// is a component of the coefficient function
					for (int i=0; i<values.Height(); i++)
						hsum += mir[i].GetWeight() * values.Row(i);
					// sum up 
					for (size_t i=0; i<dim; i++)
						AtomicAdd(sum(i), hsum(i));
				}
			);
			return sum(0);
		}
	};

	void ExportIntCoefficientFunction(py::module m);
}

#endif
